include <omdl-v0.6.1/shapes/shapes3d.scad>;
include <omdl-v0.6.1/shapes/shapes2de.scad>;
include <omdl-v0.6.1/tools/tools_utility.scad>;

$fn=32;

/*******************************************************************************
  form:   int:value

          0 = standard height
          1 = low profile

  ribs:   lst:[location, shape, divisions, positions]

          int:location      0 = top | 1 = none | 2 = bottom | 3 = top&bottom
          int:shape         0 = cylinder | 1 = ellips
          int:divisions     value
          lst:positions     [p(1), p(2), ..., p(n)] of dec:value

  fins:   int:value

          0 = none
          1 = side 1
          2 = side 2
          3 = both sides

  vents:  lst:[count, radius]

          int:count         value
          dec:radius        value

  tabs:   list:[[spec(1)], ..., [spec(n)]]

          lst:spec(n)       [y-offset(n), x-offset(n), hole-diamter(n)] of dec:value

  align:  lst:[x, y, z]

          int:x             0 = bracket outside
                            1 = bracket center
                            2 = bracket inside
                            3 = bracket tab screw
                            4 = back of tab
                            5 = pcie connector alignment slot

          int:y             0 = bracket edge1
                            1 = bracket tab edge1
                            2 = circuit board edge
                            3 = tab alignment circle
                            4 = bracket edge2

                            5 = connector opening edge1
                            6 = connector opening center
                            7 = connector opening edge1

          int:z             0 = bracket bottom
                            1 = bottom of pci connector
                            2 = bottom of board / board stop
                            3 = bottom of bracket tab
                            4 = top of bracket tab

                            5 = connector opening bottom
                            6 = connector opening center
                            7 = connector opening top

*******************************************************************************/

module pc_bracket( form, ribs, fins, vents, tabs, align )
{
  bff = defined_or(form, 0);

  //
  // reference:
  //
  // PCI Express Card Electromechanical Specification
  // Revision 1.1, March 28, 2005, pp 71-80
  //

  mth  =   0.86;                            // 20 GA

  sv01 = ciselect( [112.75, 71.46], bff);
  sv02 = ciselect( [  2.54, -2.14], bff);   // bff1: 20.73 - 18.59
  sv03 = ciselect( [ 21.59, 20.73], bff);   // bff1: 17.15 + 3.58
  sv04 = ciselect( [ 11.43, 11.84], bff);
  sv05 = 18.42;
  sv06 = ciselect( [  5.08,  6.35], bff);
  sv07 = 4.42;
  sv08 = ciselect( [ 89.90, 54.53], bff);
  sv09 = ciselect( [ 10.16,  9.04], bff);
  sv10 = ciselect( [  4.56,  0.00], bff);
  sv11 = 4.11;
  sv12 = ciselect( [120.02, 79.20], bff);

  sv13 = 59.05;
  sv14 = ciselect( [100.36, 63.58], bff);
  sv15 = ciselect( [111.15, 68.90], bff);
  sv16 = ciselect( [106.65, 64.40], bff);
  sv17 = ciselect( [ 12.06, 12.07], bff);

  sv18 = ciselect( [  2.54, -3.58], bff);

  // configuration and defaults
  brm = edefined_or(ribs, 0, 3);
  brs = edefined_or(ribs, 1, 1);
  brd = sv05/edefined_or(ribs, 2, 10);
  brc = edefined_or(ribs, 3, [3,7]);

  bfc = defined_or(fins, 0);

  vhc = edefined_or(vents, 0, 0);
  vhr = edefined_or(vents, 1, 3);
  vhs = vhr*4;

  mtt = 2.00 * mth;
  tbf = 1.25;

  // alignment offsets
  bax = ciselect(
      [
        0,
        -mth/2,
        -mth,
        sv06,
        sv04,
        -sv13,
        0
      ]
      , edefined_or(align, 0, 0) );

  bay = ciselect(
      [
        0,
        ciselect([-sv02, -sv18], bff),
        -sv05+mtt,
        ciselect([-sv05, 0], bff),
        ciselect([-sv03, -sv05], bff),
        -sv05/2+sv17/2,
        -sv05/2,
        -sv05/2-sv17/2,
        0
      ]
      , edefined_or(align, 1, 0) );

  baz = ciselect(
      [
        sv12-sv01,
        -sv01+sv14+sv15-sv16,
        -sv01+sv14,
        -sv01,
        -sv01-mth,
        -sv01+sv08+sv09,
        -sv01+sv08/2+sv09,
        -sv01+sv09,
        0
      ]
      , edefined_or(align, 2, 0) );


  //
  // bracket
  //

  translate([bax,bay,baz])
  rotate([90,0,90])
  difference()
  {
    // connector opening
    col = sv08;
    coo = sv01-col-sv09;

    union()
    {
      // top tab
      translate([0,sv01,aeps])
      rotate([270,0,0])
      difference()
      {
        vrf = ciselect( [[0,0,1.5,1.5], [1.5,0,1.5,1.5]], bff);
        translate([sv18,0,0])
        erectangle(size=[sv03-abs(sv02), sv04], h=mth, vr=vrf, center=false);

        if ( bff == 0 )
        {
          translate([sv05,sv06,-mth/2])
          union()
          {
            cylinder(d=sv07, h=mth*2);
            translate([0,-sv07/2,0])
            cube([sv07,sv07,mth*2]);
          }
        }
        else
        {
          translate([0,sv06,-mth/2])
          union()
          {
            cylinder(d=sv07, h=mth*2);
            translate([-sv07,-sv07/2,0])
            cube([sv07,sv07,mth*2]);
          }
        }
      }

      // tab to bracket offset
      toh  = sv10;
      translate([sv02,sv01-toh+mth,0])
      erectangle(size=[sv03-abs(sv02), toh], h=mth, vr=[0,toh/1.75,0,0], vrm=2, center=false);

      // tab fillet
      tbfl = ciselect( [sv03-sv02, sv03+sv02+sv18], bff);
      tbfo = ciselect( [sv02, 0], bff);
      translate([tbfo,sv01-tbf,0])
      mirror([0,1,1])
      rotate([0,90,0])
      etriangle_sas(s1=tbf,a=90,s2=tbf,h=tbfl);

      // bracket
      vrf  = ciselect( [[4.00,4.00,0,2.5], [4.00,4.00,1.5,1.5]], bff);
      vrmf = ciselect( [15, 3], bff);
      erectangle(size=[sv05, sv01-toh+mth+aeps*4], h=mth, vr=vrf, vrm=vrmf, center=false);

      // bottom tab
      btx = sv05-sv11*2;
      bty = sv12-sv01;
      translate([sv11,-bty+aeps,0])
      erectangle(size=[btx, bty], h=mth, vr=[1.5,1.5,0,0], vrm=0, center=false);

      // ribs
      for (x=brc)
      {
        translate([x*brd, coo, mth/2])
        rotate([270,0,0])
        difference()
        {
          if (brs == 0)
          {
            union()
            {
              cylinder(d=brd, h=col-brd);
              sphere(d=brd);
              translate([0,0,col-brd])
              sphere(d=brd);
            }
          }
          else
          {
            union()
            {
              eellipse(size=[1,1/2]*brd, h=col-brd);
              rotate([0,90,0])
              ellipsoid([1,2]*brd);
              translate([0,0,col-brd])
              rotate([0,90,0])
              ellipsoid([1,2]*brd);
            }
          }


          translate([-brd*2,-brd*brm,-brd]/2)
          cube([brd*2,brd,col+brd]*(1+aeps));
        }
      }

      // board mount tabs
      for (n = tabs)
      {
        ty = edefined_or(n, 0, n);    // y-offset vector[0] or scalar
        tx = edefined_or(n, 1, 9.40); // x-offset or standard
        hd = edefined_or(n, 2, 4);    // hole diameter or standard

        tt = mtt;
        tl = tx+hd;
        tw = hd*2;

        translate([sv05, sv01-tw/2-ty, -aeps])
        rotate([0,-90,0])
        difference()
        {
          union()
          {
            erectangle(size=[tl, tw], h=tt, vr=[0,hd/2,hd/2,0], center=false);
            translate([tbf+mth,0,tt])
            rotate([270,180,0])
            mirror([0,0,0])
            etriangle_sas(s1=tbf,a=90,s2=tbf,h=tw);
          }
          translate([tx, tw/2, -tt/2])
          cylinder(d=hd, h=tt*2);
        }
      }

      // edge fins
      for (x = [[0,0], [1,sv05-mth]])
      {
        if ( bitwise_is_equal(fins, first(x), 1) )
        translate([second(x),coo-mth,mth*3-aeps])
        rotate([0,90,0])
        erectangle(size=[mth*2, sv08], h=mth, vr=[1,0,0,1]*mth*1.5, center=false);
      }
    }

    // vent holes
    vho = coo + col/2 - (vhc-1)*vhs/2;
    if (vhc > 0)
    {
      for (y=[1:vhc])
      {
        translate([sv05/2, vho + (y-1)*vhs, -mth/2-mth])
        engon(n=6, r=vhr, h=mth*4, vr=vhr/2);
      }
    }
  }
}


//
// examples
//

pc_bracket_example = 6;

// example1
if (pc_bracket_example == 1)
grid_repeat(g=[1,6,2], i=[30, 30, 130])
{
  pc_bracket(form=0);
  pc_bracket(form=0, ribs=[2], fins=3, vents=[10,2.25]);
  pc_bracket(form=0, ribs=[0,1,8,[2,6]], vents=[8,2.5]);
  pc_bracket(form=0, ribs=[3,1,8,[2,3,4,5,6]], vents=[8,2.5]);
  pc_bracket(form=0, ribs=[1], fins=3, tabs=[[10.11], [10.11+85.40,19.40,5]]);
  pc_bracket(form=0, ribs=[3,0,8, [2,6]], vents=[8,2.5], tabs=[45,85]);

  pc_bracket(form=1);
  pc_bracket(form=1, ribs=[2], fins=3, vents=[5,2.25]);
  pc_bracket(form=1, ribs=[0,1,8,[2,6]], vents=[5,2.5]);
  pc_bracket(form=1, ribs=[3,1,8,[2,3,4,5,6]], vents=[4,2.5]);
  pc_bracket(form=1, ribs=[1], fins=3, tabs=[[10], [55,19.40,5]]);
  pc_bracket(form=1, ribs=[3,0,8, [2,6]], vents=[3,2.5], tabs=[20,55]);
}

// example2
if (pc_bracket_example == 2)
pc_bracket(align=[0,0,0], ribs=[2,0,6,[1.5,4.5]], vents=[8,2.5]);

// example3
if (pc_bracket_example == 3)
pc_bracket(align=[0,0,0], ribs=[2,1,8,[2,3,4,5,6]], fins=3, vents=[8,2.5]);

// example4
if (pc_bracket_example == 4)
pc_bracket(align=[0,0,0], form=1, ribs=[2,0,8,[2,3,4,5,6]]);

// example5
if (pc_bracket_example == 5)
pc_bracket(align=[0,0,0], form=1, fins=3, ribs=[3,1,8,[2,3,4,5,6]], vents=[3], tabs=[20,55]);


/*******************************************************************************
 EOF
*******************************************************************************/


difference() {
    pc_bracket(align=[0,0,0], form=0, tabs=[[42.5,11],[99]]);
    translate([-7,3,24]) cube([10,13,52]);
}


//cube([10,10,20.5]);
//cube([10,10,77]);



